/*
 * testlibusb.c
 *
 *  Test suite program
 */

#include <stdio.h>
#include <stdlib.h>
#include <usb.h>
#include <fcntl.h>
#include <unistd.h>
#include <signal.h>


enum { read_port, write_port};
enum { stop, right, left, get_pos};

int signal_terminate=0;

void sigint();

void sigint() {
  signal(SIGINT,sigint); /* reset signal */
  signal_terminate=SIGINT;
}

struct usb_device *finddev(int vendid, int prodid, int num) {
  struct usb_device *dev;
  struct usb_bus *bus;
  usb_init();
  usb_find_busses();
  usb_find_devices();

  for (bus = usb_get_busses(); bus; bus = bus->next) {
    for (dev = bus->devices; dev; dev = dev->next) {
      if ( (dev->descriptor.idVendor == vendid) && (dev->descriptor.idProduct == prodid) ) {
	if (0 == num--) {
	  return dev;
	}
      }
    }
  }
  return NULL;
}


int getlock(char *lf) {
  int fd;
  if( (fd = open(lf, O_RDWR | O_CREAT | O_EXCL)) == -1 )
    {
      fprintf(stderr, "Unable to get lockfile: %d\n",fd);
      return(-1);
    }
  return(fd);
}

int releaselock(int fd, char *lf) {
  close(fd);
  unlink(lf);
  return(0);
}


int rwport(struct usb_dev_handle *dev, int rw, int port, int value) {
  int retval;
  char iobuf[8];

  switch(rw) {
  case read_port:
    retval=usb_control_msg(dev, 0x80 | (0x02 << 5) | 0x03, 0x04,
			   port, 0, iobuf, 8, 67);
    break;
  case write_port:
    retval=usb_control_msg(dev, 0x80 | (0x02 << 5) | 0x03, 0x05,
			 port, value, iobuf, 8, 67);
    break;
  default:
    fprintf(stderr, "Wrong Command!\n");
    retval=-1;
    break;
  }
  if (retval < 0) 
    return retval;
//  if (rw==read_port)
//    fprintf(stderr, "%d %d %d %d \n", iobuf[1], ~iobuf[1], 0xff&iobuf[1], 0xff&~iobuf[1]);
  return (0xff & ~iobuf[1]);
}

int turnleft(struct usb_dev_handle *t) {
  return(rwport(t, write_port, 1, 0x02));
}


int turnright(struct usb_dev_handle *t) {
  return(rwport(t, write_port, 1, 0x08));
}

int turnstop(struct usb_dev_handle *t) {
  return(rwport(t, write_port, 1, 0x0a));
}

int readport(struct usb_dev_handle *t) {
  return (rwport(t, read_port, 0, 0));
}

int main(int argc, char **argv)
{
  struct usb_device *dev;
  struct usb_dev_handle *t;
  int pos=-1, num=0;
  int n, fd;
  int turn;

  char *lockfile= "/var/lock/cymot-0.lock";

  signal(SIGINT,sigint);

  if ( (fd=getlock(lockfile)) ==-1 )
    exit(1);

  while( (n=getopt(argc, argv, "i:lrsp:hc")) != EOF ) {
    switch (n) {
    case 'i':
      if ( !(scanf(optarg, "%i", &num))) {
	fprintf(stderr, "Wrong interface-number (integer): %s\n", optarg);
	releaselock(fd, lockfile);
	exit(1);
      }
      break;
    case 'l':
      turn=left;
      break;
    case 'r':
      turn=right;
      break;
    case 's':
      turn=stop;
      break;
    case 'p':
      if( !(sscanf(optarg, "%i", &pos)) || ((pos>255) || (0>pos))) {
	fprintf(stderr, "Wrong position (0-255): %s (%d)\n",optarg, pos);
	releaselock(fd, lockfile);
	exit(1);
      }
      break;
    case 'h':
      fprintf(stderr, "-l, -r, -s (left, right, stop), -p <0-255> (position), -i <interface-number>\n");
      fprintf(stderr, "Use only ONE parm of -l/-r/-s!\n");
      releaselock(fd, lockfile);
      exit(1);
      break;
    case 'c':
      turn=get_pos;
      break;
    default:
      fprintf(stderr, "wrong parameter, try -h\n");
      releaselock(fd, lockfile);
      exit(1);
      break;
    }	
  }
  if ( !(dev=finddev(0x0a2c, 0x0008, num)) ) {
    fprintf(stderr, "No such interface (%d)!\n", num);
    releaselock(fd, lockfile);
    exit(1);
  }
  t=usb_open(dev);
  //  usb_claim_interface(t, 0);

  //Action
  switch (turn) {
  case stop:
    turnstop(t);
    break;
  case right:
    turnright(t);
    break;
  case left:
    turnleft(t);
    break;
  case get_pos:
    fprintf(stderr, "Current position is %d\n",readport(t));
    break;
  default:
    if (pos != -1) {
      int cur, cur1=255, cur2=255, cur3=255, cur4=255, cur5=255;
//      pos=pos>>1<<1;
//      while ( ( pos > (cur=readport(t)>>1<<1 ) ) || ( pos<cur ) ) {
      while ( ( pos > (cur=readport(t) ) ) || ( pos<cur ) ) {
	fprintf(stderr, "Cur: %d, pos: %d\n", cur, pos);
	cur5=cur4;
	cur4=cur3;
	cur3=cur2;
	cur2=cur1;
	cur1=cur;
//	if ( cur == cur5 && cur==cur3) {
	if ( (cur == cur5 && cur==cur3 && cur==cur4 && cur==cur2) || (0 <= cur-pos && cur-pos < 5) || (0 >= cur-pos && cur-pos > -5) ) {
	  break;
	}
	if (cur<pos) {
	  turnright(t);
	  usleep(50000);
	  turnstop(t);
	} else {
	  turnleft(t);
	  usleep(50000);
	  turnstop(t);
	}
 	switch (signal_terminate) {
	case 0:
	  // No signal received
	  break;
	case SIGINT:
	  // We got sigkill, cleanup (for daemon mode) and exit
	  signal_terminate=0;
	  turnstop(t);
	  usb_close(t);
	  releaselock(fd, lockfile);
	  exit(1);
	  break;
	default:
	  fprintf(stderr,"Unknown signal received, ignoring.");
	  break;
	}
	usleep(20000);
      }
      turnstop(t);
      //      fprintf(stderr, "Current pos: %d, desired pos: %d\n", cur, pos);
    } else {
      fprintf(stderr, "No valid parameter specified, try -h\n");
    }
    break;
  }


  //  usb_release_interface(t, 0);
  usb_close(t);

  releaselock(fd, lockfile);

  return 0;
}


